# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'layout.ui'
#
# Created by: PyQt5 UI code generator 5.9.2
#
# WARNING! All changes made in this file will be lost!

from PyQt5 import QtCore, QtGui, QtWidgets

class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1130, 858)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(MainWindow.sizePolicy().hasHeightForWidth())
        MainWindow.setSizePolicy(sizePolicy)
        font = QtGui.QFont()
        font.setFamily("Segoe UI")
        font.setPointSize(10)
        MainWindow.setFont(font)
        MainWindow.setWindowTitle("LOFAR Solar Beam-formed Inspector")
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.mplw = mplw(self.centralwidget)
        self.mplw.setGeometry(QtCore.QRect(20, 10, 791, 541))
        self.mplw.setObjectName("mplw")
        self.pointing = QtWidgets.QPushButton(self.centralwidget)
        self.pointing.setGeometry(QtCore.QRect(980, 20, 121, 61))
        self.pointing.setObjectName("pointing")
        self.button_gen = QtWidgets.QPushButton(self.centralwidget)
        self.button_gen.setGeometry(QtCore.QRect(840, 20, 121, 61))
        self.button_gen.setObjectName("button_gen")
        self.verticalLayoutWidget = QtWidgets.QWidget(self.centralwidget)
        self.verticalLayoutWidget.setGeometry(QtCore.QRect(840, 110, 261, 91))
        self.verticalLayoutWidget.setObjectName("verticalLayoutWidget")
        self.verticalLayout_4 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget)
        self.verticalLayout_4.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_4.setObjectName("verticalLayout_4")
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label.setAlignment(QtCore.Qt.AlignCenter)
        self.label.setObjectName("label")
        self.horizontalLayout_6.addWidget(self.label)
        self.input_t = QtWidgets.QLineEdit(self.verticalLayoutWidget)
        self.input_t.setObjectName("input_t")
        self.horizontalLayout_6.addWidget(self.input_t)
        self.horizontalLayout_6.setStretch(0, 3)
        self.verticalLayout_4.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_2 = QtWidgets.QLabel(self.verticalLayoutWidget)
        self.label_2.setAlignment(QtCore.Qt.AlignCenter)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_7.addWidget(self.label_2)
        self.input_f = QtWidgets.QLineEdit(self.verticalLayoutWidget)
        self.input_f.setObjectName("input_f")
        self.horizontalLayout_7.addWidget(self.input_f)
        self.horizontalLayout_7.setStretch(0, 3)
        self.verticalLayout_4.addLayout(self.horizontalLayout_7)
        self.frame = QtWidgets.QFrame(self.centralwidget)
        self.frame.setGeometry(QtCore.QRect(840, 220, 261, 151))
        self.frame.setFrameShape(QtWidgets.QFrame.Box)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayoutWidget_2 = QtWidgets.QWidget(self.frame)
        self.verticalLayoutWidget_2.setGeometry(QtCore.QRect(50, 40, 211, 111))
        self.verticalLayoutWidget_2.setObjectName("verticalLayoutWidget_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_2)
        self.verticalLayout.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout.setObjectName("verticalLayout")
        self.interp_nearest = QtWidgets.QRadioButton(self.verticalLayoutWidget_2)
        self.interp_nearest.setObjectName("interp_nearest")
        self.verticalLayout.addWidget(self.interp_nearest)
        self.interp_linear = QtWidgets.QRadioButton(self.verticalLayoutWidget_2)
        self.interp_linear.setObjectName("interp_linear")
        self.verticalLayout.addWidget(self.interp_linear)
        self.interp_cubic = QtWidgets.QRadioButton(self.verticalLayoutWidget_2)
        self.interp_cubic.setObjectName("interp_cubic")
        self.verticalLayout.addWidget(self.interp_cubic)
        self.label_4 = QtWidgets.QLabel(self.frame)
        self.label_4.setGeometry(QtCore.QRect(10, 10, 241, 16))
        self.label_4.setObjectName("label_4")
        self.frame_2 = QtWidgets.QFrame(self.centralwidget)
        self.frame_2.setGeometry(QtCore.QRect(840, 390, 261, 151))
        self.frame_2.setFrameShape(QtWidgets.QFrame.Box)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.verticalLayoutWidget_3 = QtWidgets.QWidget(self.frame_2)
        self.verticalLayoutWidget_3.setGeometry(QtCore.QRect(50, 40, 211, 111))
        self.verticalLayoutWidget_3.setObjectName("verticalLayoutWidget_3")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.verticalLayoutWidget_3)
        self.verticalLayout_2.setContentsMargins(0, 0, 0, 0)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.show_disk = QtWidgets.QCheckBox(self.verticalLayoutWidget_3)
        self.show_disk.setObjectName("show_disk")
        self.verticalLayout_2.addWidget(self.show_disk)
        self.show_FWHM = QtWidgets.QCheckBox(self.verticalLayoutWidget_3)
        self.show_FWHM.setObjectName("show_FWHM")
        self.verticalLayout_2.addWidget(self.show_FWHM)
        self.show_peak = QtWidgets.QCheckBox(self.verticalLayoutWidget_3)
        self.show_peak.setObjectName("show_peak")
        self.verticalLayout_2.addWidget(self.show_peak)
        self.label_3 = QtWidgets.QLabel(self.frame_2)
        self.label_3.setGeometry(QtCore.QRect(10, 10, 241, 16))
        self.label_3.setObjectName("label_3")
        self.log = QtWidgets.QTextBrowser(self.centralwidget)
        self.log.setGeometry(QtCore.QRect(20, 560, 1081, 191))
        font = QtGui.QFont()
        font.setFamily("Consolas")
        self.log.setFont(font)
        self.log.setObjectName("log")
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 1130, 26))
        self.menubar.setObjectName("menubar")
        self.menufile = QtWidgets.QMenu(self.menubar)
        self.menufile.setObjectName("menufile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.loadsav = QtWidgets.QAction(MainWindow)
        self.loadsav.setObjectName("loadsav")
        self.loadfits = QtWidgets.QAction(MainWindow)
        self.loadfits.setObjectName("loadfits")
        self.actionClear = QtWidgets.QAction(MainWindow)
        self.actionClear.setObjectName("actionClear")
        self.loadcube = QtWidgets.QAction(MainWindow)
        self.loadcube.setObjectName("loadcube")
        self.menufile.addAction(self.loadsav)
        self.menufile.addAction(self.loadcube)
        self.menufile.addAction(self.loadfits)
        self.menufile.addAction(self.actionClear)
        self.menubar.addAction(self.menufile.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        self.pointing.setText(_translate("MainWindow", "Pointing"))
        self.button_gen.setText(_translate("MainWindow", "Beam Form"))
        self.label.setText(_translate("MainWindow", "Time"))
        self.label_2.setText(_translate("MainWindow", "Frequency"))
        self.interp_nearest.setText(_translate("MainWindow", "Nearest"))
        self.interp_linear.setText(_translate("MainWindow", "Linear"))
        self.interp_cubic.setText(_translate("MainWindow", "Cubic"))
        self.label_4.setText(_translate("MainWindow", "Interpolation Method"))
        self.show_disk.setText(_translate("MainWindow", "Show Solar Disk"))
        self.show_FWHM.setText(_translate("MainWindow", "Show FWHM"))
        self.show_peak.setText(_translate("MainWindow", "Show Peak"))
        self.label_3.setText(_translate("MainWindow", "Actions"))
        self.log.setHtml(_translate("MainWindow", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:\'Consolas\'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><br /></p></body></html>"))
        self.menufile.setTitle(_translate("MainWindow", "File"))
        self.loadsav.setText(_translate("MainWindow", "Load .sav data"))
        self.loadfits.setText(_translate("MainWindow", "Load .fits data"))
        self.actionClear.setText(_translate("MainWindow", "Clear"))
        self.loadcube.setText(_translate("MainWindow", "Load cube.sav data"))

from mplw import mplw
import resource_rc
